#include "calculator.hpp"

Calculator::Calculator(QWidget *qw, QLineEdit *qle)
: QObject(), mainWidget(qw), edit(qle) { }

void Calculator::push()
{
	QStringList list = edit->text().split("/");

	if (list.length() != 2)
	{
		QMessageBox::warning(	mainWidget,
					"WARNING",
					"Falsches Eingabeformat!\nRichtiges Format: Zahl/Zahl",
					QMessageBox::Ok,
					QMessageBox::Ok);
		return;
	}

	bool ok0, ok1;

	int z = list[0].toInt(&ok0);
	int n = list[1].toInt(&ok1);

	if (ok0 == true && ok1 == true)
	{
		try
		{
			mystack.push(Rational(z, n));
		}
		catch (const std::exception& e)
		{
			QMessageBox::warning(	mainWidget,
						"WARNING",
						e.what(),
						QMessageBox::Ok,
						QMessageBox::Ok);
			return;
		}

		emit numberOfElementsChanged((int) mystack.size());
		emit topOfStackChanged(mystack.top().toString());
	}
	else
	{
		QMessageBox::warning(	mainWidget,
					"WARNING",
					"Falsches Eingabeformat!\nRichtiges Format: Zahl/Zahl",
					QMessageBox::Ok,
					QMessageBox::Ok);
	}
}

void Calculator::add()
{
	if (mystack.size() < 2)
	{
		QMessageBox::warning(	mainWidget,
					"WARNING",
					"Es muessen mindestens 2 Elemente\nauf dem Stack liegen!",
					QMessageBox::Ok,
					QMessageBox::Ok);
		return;
	}

	Rational top1 = mystack.top();
	mystack.pop();
	Rational top2 = mystack.top();
	mystack.pop();

	mystack.push(top1 + top2);

	emit numberOfElementsChanged((int) mystack.size());
	emit topOfStackChanged(mystack.top().toString());
}

void Calculator::mul()
{
	if (mystack.size() < 2)
	{
		QMessageBox::warning(	mainWidget,
					"WARNING",
					"Es muessen mindestens 2 Elemente\nauf dem Stack liegen!",
					QMessageBox::Ok,
					QMessageBox::Ok);
		return;
	}

	Rational top1 = mystack.top();
	mystack.pop();
	Rational top2 = mystack.top();
	mystack.pop();

	mystack.push(top1 * top2);

	emit numberOfElementsChanged((int) mystack.size());
	emit topOfStackChanged(mystack.top().toString());
}

void Calculator::addInv()
{
	if (mystack.size() < 1)
	{
		QMessageBox::warning(	mainWidget,
					"WARNING",
					"Es muss mindestens 1 Element\nauf dem Stack liegen!",
					QMessageBox::Ok,
					QMessageBox::Ok);
		return;
	}

	Rational top = mystack.top();
	top.setZaehler(top.getZaehler() * -1);

	mystack.pop();
	mystack.push(top);

	emit topOfStackChanged(mystack.top().toString());
}

void Calculator::mulInv()
{
	if (mystack.size() < 1)
	{
		QMessageBox::warning(	mainWidget,
					"WARNING",
					"Es muss mindestens 1 Element\nauf dem Stack liegen!",
					QMessageBox::Ok,
					QMessageBox::Ok);
		return;
	}

	Rational top = mystack.top();
	int tmp = top.getZaehler();
	top.setZaehler(top.getNenner());

	try
	{
		top.setNenner(tmp);
	}
	catch(const std::exception& e)
	{
		QMessageBox::warning(	mainWidget,
					"WARNING",
					e.what(),
					QMessageBox::Ok,
					QMessageBox::Ok);
		return;
	}

	if (top.getNenner() < 0)
	{
		top.setZaehler(top.getZaehler() * -1);
		top.setNenner(top.getNenner() * -1);
	}

	mystack.pop();
	mystack.push(top);

	emit topOfStackChanged(mystack.top().toString());
}
