#include "rational.hpp"

Rational::Rational(int z, int n)
: zaehler(z), nenner(std::fabs(n))
{
	if (n < 0) zaehler *= -1;

	pruefeNenner();
}

int Rational::getZaehler()
{
	return zaehler;
}

int Rational::getNenner()
{
	return nenner;
}

void Rational::setZaehler(int z)
{
	zaehler = z;
}

void Rational::setNenner(int n)
{
	nenner = n;

	pruefeNenner();
}

std::string Rational::toString() const
{
	std::stringstream strstr;
	strstr << zaehler << "/" << nenner;
	return strstr.str();
}

void Rational::pruefeNenner() const
{
	if(this->nenner == 0)
	{
		std::stringstream strstr;
		strstr << *this << ": Nenner darf nicht Null sein!";
		throw std::invalid_argument(strstr.str());
	}
}

int Rational::ggT(int a, int b) const
{
	if (a == 0) return b;

	while (b != 0)
	{
		if (a > b) a = a - b;
		else       b = b - a;
	}

	return a;
}

int Rational::kgV(int a, int b) const
{
	return std::fabs(a * b) / Rational::ggT(a, b);
}

Rational Rational::operator*(const Rational& rechts) const
{
	return Rational(this->zaehler * rechts.zaehler, this->nenner * rechts.nenner);
}

Rational& Rational::operator*=(const Rational& rechts)
{
	this->zaehler *= rechts.zaehler;
	this->nenner *= rechts.nenner;
	return *this;
}

Rational Rational::operator+(const Rational& rechts) const
{
	int nenner = Rational::kgV(this->nenner, rechts.nenner);
	int zaehler1 = (nenner/this->nenner) * this->zaehler;
	int zaehler2 = (nenner/rechts.nenner) * rechts.zaehler;
	return Rational(zaehler1 + zaehler2, nenner);
}

Rational& Rational::operator+=(const Rational& rechts)
{
	int nenner = Rational::kgV(this->nenner, rechts.nenner);
	int zaehler1 = (nenner/this->nenner) * this->zaehler;
	int zaehler2 = (nenner/rechts.nenner) * rechts.zaehler;
	this->zaehler = zaehler1 + zaehler2;
	this->nenner = nenner;
	return *this;
}

Rational Rational::operator-(const Rational& rechts) const
{
	int nenner = Rational::kgV(this->nenner, rechts.nenner);
	int zaehler1 = (nenner/this->nenner) * this->zaehler;
	int zaehler2 = (nenner/rechts.nenner) * rechts.zaehler;
	return Rational(zaehler1 - zaehler2, nenner);
}

Rational& Rational::operator-=(const Rational& rechts)
{
	int nenner = Rational::kgV(this->nenner, rechts.nenner);
	int zaehler1 = (nenner/this->nenner) * this->zaehler;
	int zaehler2 = (nenner/rechts.nenner) * rechts.zaehler;
	this->zaehler = zaehler1 - zaehler2;
	this->nenner = nenner;
	return *this;

}

Rational Rational::operator/(const Rational& rechts) const
{
	return Rational(this->zaehler * rechts.nenner, this->nenner * rechts.zaehler);
}

Rational& Rational::operator/=(const Rational& rechts)
{
	this->zaehler *= rechts.nenner;
	this->nenner *= rechts.zaehler;

	if (this->nenner < 0)
	{
		this->zaehler *= -1;
		this->nenner = std::fabs(this->nenner);
	}

	pruefeNenner();

	return *this;
}

bool Rational::operator==(const Rational& rechts) const
{
	return (this->zaehler * rechts.nenner) == (rechts.zaehler * this->nenner);
}

bool Rational::operator!=(const Rational& rechts) const
{
	return (this->zaehler * rechts.nenner) != (rechts.zaehler * this->nenner);
}

bool Rational::operator<(const Rational& rechts) const
{
	return (this->zaehler * rechts.nenner) < (rechts.zaehler * this->nenner);
}

bool Rational::operator<=(const Rational& rechts) const
{
	return (this->zaehler * rechts.nenner) <= (rechts.zaehler * this->nenner);
}

bool Rational::operator>(const Rational& rechts) const
{
	return (this->zaehler * rechts.nenner) > (rechts.zaehler * this->nenner);
}

bool Rational::operator>=(const Rational& rechts) const
{
	return (this->zaehler * rechts.nenner) >= (rechts.zaehler * this->nenner);
}

std::ostream& operator<<(std::ostream& os, const Rational& r)
{
	return (os << r.toString());
}
